<?php

class DBI
{
    /**
     * @var DBD_mysqli
     */
    public $DBD;
    /**
     * @var object
     */
    public $CONN;
    public $CONFIG;
    public $DBTYPE;
    public $prepare = "";
    public $row = array();
    public $db_host;
    public $db_datenbank;
    public $db_user;
    public $db_passwort;
    /**
     * @param boolean $debug
     */
    public $debug = false;
    public $dbsocket;
    public $dbhost;
    public $dbname;
    public $dbuser;
    public $dbpwd;
    public $cachedir;
    public $dbport = 3306;

    /**
     * show the error and make exit
     * @param string $str
     */
    function error($str)
    {
        echo "$str<BR>\n";
        exit();
    }

    function __construct($db_host, $db_datenbank, $db_user, $db_passwort, $cachedir)
    {
        $this->dbsocket = false;
        $this->dbhost = $db_host;
        $this->dbname = $db_datenbank;
        $this->dbuser = $db_user;
        $this->dbpwd = $db_passwort;
        $this->cachedir = $cachedir;
        $this->DBTYPE = 'mysqli';

        $DBDClass = 'DBD_' . $this->DBTYPE;

        $this->DBD = new $DBDClass($this);
    }

    /**
     * Open a connection to a Database Server
     * @return boolean
     */
    function connect()
    {
//    var_dump($this->DBTYPE );exit;
        if ($this->DBTYPE == 'mysql') {
            $this->CONN = $this->DBD->connect($this->dbname, ((strlen($this->dbsocket)) ? ":" . $this->dbsocket : $this->dbhost . ":" . $this->dbport), $this->dbuser, $this->dbpwd);
        } else {
            $this->CONN = $this->DBD->connect($this->dbname, $this->dbhost, $this->dbuser, $this->dbpwd, $this->dbport, (strlen($this->dbsocket) ? $this->dbsocket : false));
        }
        return true;
    }

    /**
     * Close the connection to a Database Server
     */
    function disconnect()
    {
        if ($this->DBD) {
            $this->DBD->disconnect();
        }
    }

    function prepare($query, $cache = 0)
    {
        $sth = $this->DBD->prepare($query, $cache);
        return ($sth);
    }

    function escape($string)
    {
        if (!$this->CONN) {
            $this->connect();
        }
        return $this->DBD->escape($string);
    }

    function __destruct()
    {
        $this->disconnect();
    }
}

/**
 * @todo make new DBD_mysqli ...
 */
class DBD_mysqli
{
    /**
     * @var mixed
     */
    public $CONFIG;
    /**
     * @var
     */
    public $DBI;
    /**
     * @var mysqli
     */
    public $dbh;

    function __construct($pConfig = false)
    {
        if ($pConfig !== false) {
            $this->CONFIG = $pConfig;
        }
    }

    /**
     * @param $str
     * @return void
     */
    function error($str): void
    {
        echo($str);
    }

    /**
     * @param string $dbname
     * @param string $dbhost
     * @param string $dbuser
     * @param string $dbpwd
     * @param int $dbport
     * @param bool $dbsocket
     * @return $this|DBD_mysqli|void
     */
    function connect(string $dbname, string  $dbhost = '', string $dbuser = '', string  $dbpwd = '', int $dbport = 3306, bool $dbsocket = false)
    {
        if ($dbsocket === false)
            $conn = mysqli_connect($dbhost, $dbuser, $dbpwd, $dbname, $dbport);
        else
            $conn = mysqli_connect($dbhost, $dbuser, $dbpwd, $dbname, $dbport, $dbsocket);

        if (!$conn) {
            $this->error("DBD::mysqli Error: Could not connect to the database.\nError: " . mysqli_connect_error());
            return;
        }

        $this->dbh = $conn;

        return ($this);
    }

    function disconnect()
    {
        if ($this->dbh) {
            mysqli_close($this->dbh);
        }
        unset ($this->dbh);

        return (1);
    }

    function escape($string)
    {
        return (mysqli_real_escape_string($this->dbh, $string));
    }

    function prepare($query, $cache = 0)
    {
        $sth = new STH_mysqli ();
        $sth->DBH($this);
        $sth->prepare($query, $cache);
        return ($sth);
    }
}

class STH_mysqli
{
    public $query;
    public $DBH;
    public $dbh;
    public $result;
    public $cache;

    function error($str)
    {
        echo "$str<BR>\n";
        exit ();
    }

    function DBH(&$dbh)
    {
        $this->DBH = &$dbh;
        $this->dbh = &$dbh->dbh;
        return ($dbh);
    }

    function prepare($query, $cache)
    {
        if ($cache && preg_match("%^SELECT.+%i", trim($query))) {
            $this->cache ['sth'] = new CACHE ($cache);
            $this->cache ['file'] = "../cache/" . md5($query);
        }
        $this->query = $query;
        return ($query);
    }

    function execute()
    {
        if (is_array($this->cache)) {

            $this->cache ['data'] ['row'] ['array'] = array();
            $this->cache ['data'] ['row'] ['hash'] = array();
            $this->cache ['data'] ['row'] ['hashref'] = array();

            if ($this->cache ['sth']->read_cache_file($this->cache ['file'], $this->cache ['data'])) {

                $this->cache ['data'] = unserialize(base64_decode($this->cache ['data']));
                $this->result = $this->cache ['data'] ['result'];
            }
            if (!$this->result) {

                $this->cache ['init'] = true;
                if (!$this->dbh) {
                    $this->DBH->connect($this->DBH->CONFIG->dbname, $this->DBH->CONFIG->dbhost, $this->DBH->CONFIG->dbuser, $this->DBH->CONFIG->dbpwd, $this->DBH->CONFIG->dbport, (strlen($this->DBH->CONFIG->dbsocket) ? $this->DBH->CONFIG->dbsocket : false));
                }
                $this->result = @mysqli_query($this->dbh, $this->query);

                $this->cache ['data'] ['result'] = ($this->result) ? 1 : 0;
            }
        } else {
            if (!$this->dbh) {
                $this->DBH->connect($this->DBH->CONFIG->dbname, $this->DBH->CONFIG->dbhost, $this->DBH->CONFIG->dbuser, $this->DBH->CONFIG->dbpwd, $this->DBH->CONFIG->dbport, (strlen($this->DBH->CONFIG->dbsocket) ? $this->DBH->CONFIG->dbsocket : false));
            }
            $this->result = mysqli_query($this->dbh, $this->query);
        }

        if (!$this->result) {
            return (0);
        }

        return (1);
    }

    function insert_id()
    {
        return (mysqli_insert_id($this->dbh));
    }

    function finish()
    {
        if (!$this->result) {
            return;
        }

        if (!is_array($this->cache) || isset ($this->cache ['init'])) {

            mysqli_free_result($this->result);

            if (isset ($this->cache ['init'])) {

                $this->cache ['sth']->write_cache_file($this->cache ['file'], $xxx = base64_encode(serialize($this->cache ['data'])));
            }
        }
        unset ($this->result);

        return (1);
    }

    function fields()
    {
        $fields = 0;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $fields = @mysqli_num_fields($this->result);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['fields'] = $fields;
            }
        } elseif (isset ($this->cache ['data'] ['fields'])) {
            $fields = $this->cache ['data'] ['fields'];
        }

        return $fields;
    }

    function rows()
    {
        $rows = 0;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $rows = mysqli_num_rows($this->result);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['rows'] = $rows;
            }
        } elseif (isset ($this->cache ['data'] ['rows'])) {
            $rows = $this->cache ['data'] ['rows'];
        }
        return $rows;
    }

    function affected()
    {
        $rows = 0;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $rows = @mysqli_affected_rows($this->dbh);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['affected'] = $rows;
            }
        } elseif (isset ($this->cache ['data'] ['affected'])) {
            $rows = $this->cache ['data'] ['affected'];
        }
        return $rows;
    }

    function fetchrow_array()
    {
        if (!$this->result) {
            return;
        }
        $row = false;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $row = mysqli_fetch_array($this->result);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['row'] ['array'] [] = $row;
            }
        } elseif (isset ($this->cache ['data'] ['row'] ['array'])) {
            $row = array_shift($this->cache ['data'] ['row'] ['array']);
        }
        return ($row);
    }

    function fetchrow_hash()
    {
        if (!$this->result) {
            return;
        }
        $row = false;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $row = mysqli_fetch_assoc($this->result);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['row'] ['hash'] [] = $row;
            }
        } elseif (isset ($this->cache ['data'] ['row'] ['hash'])) {
            $row = array_shift($this->cache ['data'] ['row'] ['hash']);
        }
        return ($row);
    }

    function fetchrow_hashref()
    {
        if (!$this->result) {
            return;
        }
        $row = false;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $row = mysqli_fetch_object($this->result);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['row'] ['hashref'] [] = $row;
            }
        } elseif (isset ($this->cache ['data'] ['row'] ['hashref'])) {
            $row = array_shift($this->cache ['data'] ['row'] ['hashref']);
        }
        return ($row);
    }

    function fetchrow_field()
    {
        if (!$this->result) {
            return;
        }
        $row = false;
        if (!is_array($this->cache) || isset ($this->cache ['init'])) {
            $row = mysqli_fetch_field($this->result);
            if (isset ($this->cache ['init'])) {
                $this->cache ['data'] ['row'] ['field'] [] = $row;
            }
        } elseif (isset ($this->cache ['data'] ['row'] ['field'])) {
            $row = array_shift($this->cache ['data'] ['row'] ['field']);
        }
        return ($row);
    }
}

class CACHE
{
    public $debug = false;

    /**
     * @param number $pExpires
     * @param string $debug
     */
    function __construct($pExpires = 30, $debug = false)
    {
        $this->expires = $pExpires;
        $this->debug = $debug;
    }

    function write_cache_file($filename, $data)
    {
        $result = false;
        $fp = @fopen($filename, "w");
        if ($fp) {
            if (!@flock($fp, LOCK_EX | LOCK_NB))
                @fclose($fp);
        }
        if ($fp) {
            @fputs($fp, $data);
            @fclose($fp);
            $result = true;
        }
        if ($result) {
            if ($this->debug)
                echo "OK ($filename)";
        } else {
            if ($this->debug)
                echo "Failed ($filename)";
        }
        return $result;
    }

    function read_cache_file($filename, &$data)
    {
        $result = false;
        if (!$this->expire_cache_file($filename)) {
            $fp = '';
            if (@is_readable($filename))
                $fp = @fopen($filename, "r");

            if ($fp) {
                if (!@flock($fp, LOCK_SH | LOCK_NB))
                    @fclose($fp);
            }

            if ($fp) {
                $data = "";
                while (!@feof($fp)) {
                    $data .= @fread($fp, 131072);
                }
                @fclose($fp);
                $result = true;
            }
        }
        if ($result) {
            if ($this->debug)
                echo "OK ($filename)";
        }
        return $result;
    }

    function expire_cache_file($filename)
    {
        $result = false;
        if (@is_readable($filename)) {
            $mtime = @filemtime($filename);
            $act_time = @time();
            if ($mtime < $act_time - $this->expires) {
                @unlink($filename);
                $result = true;
                if ($this->debug)
                    echo "Expired ($filename)";
            }
        } else {
            if ($this->debug)
                echo "Not exists ($filename)";
        }
        return $result;
    }
}

?>