<?php

/*
 * -----------------------------------------
 * Das Script kann unter Verwendung dieses Vermerks uneingeschr&auml;nkt
 * genutzt / ver&auml;ndert werden. ? www.bildergalerie-script.de
 * -----------------------------------------
 * Bildergalerie Script Version 3.10 - 4.11.2023
 * -----------------------------------------
 * @author 	Nico Schubert (info@schubertmedia.de)
 * @copyright 	Copyright 2011 - 2023
 * @version 	3.10 (PHP8.2)
 * -----------------------------------------
 */

class image_upload
{
    /**
     * @var array
     * @access public
     */
    public array $alowfile = array();
    /**
     * @var string
     * @access public
     */
    public string $pfad;
    /**
     * @var string
     * @access public
     */
    public string $files;
    /**
     * @var string
     * @access private
     */
    private string $fehler;
    /**
     * @var number
     * @access private
     */
    private number|int $maxsize = 1073741824;
    /**
     * @var string
     * @access private
     */
    private string $url = '';
    /**
     * @var string
     * @access private
     */
    private string $beschreibung;
    private string $rmdir_dir;
    private array $kategorie;
    private int $kategorie_id;
    private string $newdir;
    protected string $olddir;

    /**
     * @param string|array $alowfile
     */
    function set_typforupload(string|array $alowfile): void
    {
        if (is_array($alowfile))
            $this->alowfile = $alowfile;
        else
            $this->alowfile = array();
    }

    /**
     * @return boolean
     */
    function check_upload(): bool
    {
        $check_upload = true;
        if (strtolower(@ini_get('file_uploads')) == 'off' || @ini_get('file_uploads') == 0) {
            $check_upload = false;
        }
        return $check_upload;
    }

    /**
     * @param string $pfad
     * @return boolean
     */
    function setpfad(string $pfad): bool
    {
        if ($pfad != '') {
            $this->pfad = $pfad;
            return true;
        }
        return false;

    }

    /**
     * @param string $newdir
     * @return boolean
     */
    function setnewdir(string $newdir): bool
    {
        if ($newdir != '') {
            $this->newdir = $newdir;
            return true;
        }
        return false;
    }

    /**
     * @param string $olddir
     * @return bool
     */
    function setolddir(string $olddir): bool
    {
        if ($olddir != '') {
            $this->olddir = $olddir;
            return true;
        }
        return false;

    }

    /**
     * @param string $rmdir_dir
     * @return bool
     */
    function setrmdir_dir(string $rmdir_dir)
    {
        if ($rmdir_dir != '') {
            $this->rmdir_dir = $rmdir_dir;
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param int $kategorie_id
     * @return bool
     */
    function setkategorie_id(int $kategorie_id): bool
    {
        if ($kategorie_id != '') {
            $this->kategorie_id = $kategorie_id;
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param array $kategorie
     * @return boolean
     */
    function setkategorie(array $kategorie): bool
    {
        if (is_array($kategorie) and count($kategorie) != 0) {
            $this->kategorie = $kategorie;
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param string $beschreibung
     * @return boolean
     */
    function setbeschreibung(string $beschreibung): bool
    {
        if ($beschreibung != '') {
            $this->beschreibung = $beschreibung;
            return true;
        } else {
            return false;
        }
    }

    /**
     * @param int $maxsize
     * @return bool
     */
    private function setmaxsize(int $maxsize): bool
    {
        if ($maxsize != '') {
            $this->maxsize = $maxsize;
        }
        return false;
    }

    /**
     * Die Function get_beschreibung($filename); erstellt aus dem Datennamen die Beschreibung und gibt diese als String zurck
     * @param string $filename - Pfad zum Bild mit Datename und Dateiendung
     * @return string
     */
    public function get_beschreibung(string $filename = ''): string
    {
        if ($filename == '') return '';
        $info = pathinfo($filename);
        return ucfirst(basename($filename, '.' . $info['extension']));
    }

    /**
     * Gib ein Array zur?ck mit width, height, type und attr. Wenn es die Datei nicht gibt, wird ein False zur?ck gegeben
     * @param string $filename
     * @return boolean|array
     */
    public function get_imagesize(string $filename = ''): bool|array
    {
        if ($filename == '') return false;
        if (!file_exists($filename)) return false;
        list($width, $height, $type, $attr) = @getimagesize($filename);
        return array('width' => $width, 'height' => $height, 'type' => $type, 'attr' => $attr);
    }

    /**
     * @return boolean|string
     */
    function view_kategorie(): bool|string
    {
        if (!$this->kategorie) return false;
        $ausgabe = '<select name="kategorie">';
        foreach ($this->kategorie as $k => $v) {
            $ausgabe .= '<option value="' . $k . '" ' . (($this->kategorie_id == $k) ? 'selected' : '') . '>' . $v . '</option>';
        }
        $ausgabe .= '</select>';
        return $ausgabe;
    }

    /**
     * @return float|int
     */
    private function max_upload(): float|int
    {
        $max_upload = 0;
        if ($dateigroesse = @ini_get('upload_max_filesize')) {
            $max_upload = $this->phpiniwertumwandeln($dateigroesse);
        }
        if ($postgroesse = @ini_get('post_max_size')) {
            $postgroesse = $this->phpiniwertumwandeln($postgroesse);
            if ($postgroesse < $max_upload) {
                $max_upload = $postgroesse;
            }
        }
        if ($max_upload > $this->maxsize)
            $max_upload = $this->maxsize;
        return $max_upload;
    }

    /**
     * @param number $size
     * @param int $nachkommastellen
     * @return string
     */
    private function size_convert($size, int $nachkommastellen = 0)
    {
        switch ($size) {
            case ($size >= '1073741824'):
                return round($size / (1073741824), $nachkommastellen) . " GB";
                break;
            case ($size >= '1048576'):
                return round($size / (1048576), $nachkommastellen) . " MB";
                break;
            case ($size >= '1024'):
                return round($size / (1024), $nachkommastellen) . " KB";
                break;
            default:
                return $size . " Byte";
                break;
        }
    }

    /**
     * @param number $groesse
     * @return float|int
     */
    private function phpiniwertumwandeln($groesse): float|int
    {
        $werte['MB'] = 1048576;
        $werte['Mb'] = 1048576;
        $werte['M'] = 1048576;
        $werte['m'] = 1048576;
        $werte['KB'] = 1024;
        $werte['Kb'] = 1024;
        $werte['K'] = 1024;
        $werte['k'] = 1024;
        foreach ($werte as $schluessel => $wert) {
            if ((strlen($groesse) > strlen($schluessel)) && (str_ends_with($groesse, $schluessel))) {
                $groesse = substr($groesse, 0, strlen($groesse) - strlen($schluessel)) * $wert;
                break;
            }
        }


        return $groesse;
    }

    /**
     * @return boolean
     */
    function imageupload(): bool
    {
        $verzeichnis = realpath($this->pfad);
        if (is_dir($verzeichnis) && is_writeable($verzeichnis)) {
            $this->pfad = $verzeichnis . "/";
            return true;
        } else {
            $this->error(0, "Bitte bearbeiten Sie die config.php ");
            return false;
        }
    }

    /**
     * @param int $key
     * @param string $details
     */
    function error(int $key, string $details = ""): void
    {
        $mess[] = "Es fehlt ein Tempverzeichnis!";
        $mess[] = "Die Datei ist zu gro&szlig;!";
        $mess[] = "Falscher Dateityp";
        $mess[] = "Falsche Schreibrechte";
        $mess[] = "Die Datei gibt es schon in dieser Kategorie. ";
        $mess[] = "Der Ordner f&uuml;r die Kategorie gibt es nicht. ";
        $this->fehler = "Fehler: " . $mess[$key] . "<br>" . $details;
    }

    /**
     * Gibt die Fehlermeldung zurck
     * @return string
     */
    function get_error(): string
    {
        return $this->fehler ?? '';
    }

    /**
     * @return string
     */
    function upload_max_filesize(): string
    {
        $upload_max_filesize = $this->max_upload();
        return $this->size_convert($upload_max_filesize);
    }

    /**
     * @return string
     */
    function _htmlpostfile(): string
    {
        $ausgabe = '';
        $kategorie = $this->view_kategorie();
        if (!$kategorie) {
            $ausgabe = 'Bitte tragen Sie eine Kategorie ein!';
        } else {
            $upload_max_filesize = $this->max_upload();
            $ausgabe .= '<p><form action="' . $this->url . '?' . SID . '" method="post" enctype="multipart/form-data" id="uploadtag">';
            $ausgabe .= '<table class="ps" cellpadding="0" cellspacing="0" width="100%">';
            $ausgabe .= '<tr>';
            $ausgabe .= '	<td class="psh">Datei</td>';
            $ausgabe .= '	<td>';
            $ausgabe .= '		<input type="file" name="this_file" id="this_file" size="40" />';
            $ausgabe .= '	<td>';
            $ausgabe .= '</tr>';
            $ausgabe .= '<tr>';
            $ausgabe .= '	<td align="left" valign="top" class="psht">Beschreibung</td>';
            $ausgabe .= '	<td>';
            $ausgabe .= '		<textarea name="beschreibung" cols="40" rows="5" ></textarea>';
            $ausgabe .= '	<td>';
            $ausgabe .= '</tr>';
            $ausgabe .= '<tr>';
            $ausgabe .= '	<td class="psh">Kategorie</td>';
            $ausgabe .= '	<td>';
            $ausgabe .= '		' . $kategorie;
            $ausgabe .= '	<td>';
            $ausgabe .= '</tr>';
            $ausgabe .= '</table><br><input type="submit" name="send_file" value="Datei hochladen" />';
            if ($upload_max_filesize != '')
                $ausgabe .= '<input type="hidden" name="MAX_FILE_SIZE" value="' . $upload_max_filesize . '" />';
            $ausgabe .= "</form></p>";
        }
        return $ausgabe;
    }

    /**
     * Formatiert ein String zu ein Dateiname ohne Sonderzeichen
     * @param string $string
     * @return string
     */
    static function key_url(string $string): string
    {
        $string = rtrim($string);
        $map = array(
            '' => 'ae',
            '' => 'ae',
            '' => 'ss',
            '' => 'oe',
            '' => 'oe',
            '' => 'ue',
            '' => 'ue',
            '-' => ',',
            ' ' => ',',
            '' => 'a',
            '' => 'e',
            '' => 'e',
        );
        $string = str_replace(array_keys($map), array_values($map), $string);
        $string = strtolower($string);
        $string = preg_replace('#[^a-z0-9,\.]#', '', $string);
        return substr(str_replace(",,", ",", $string), 0, 100);
    }

    /**
     * @return boolean
     */
    function create_dir(): bool
    {
        if ($this->pfad != '' and $this->newdir != '') {
            $dir = $this->pfad . '/bilder/' . image_upload::key_url($this->newdir);
            if (@is_dir($dir) === false) {
                return @mkdir($dir, 0777);
            }
        }
        return false;
    }

    /**
     * @return bool
     */
    function change_dir(): bool
    {
        if ($this->pfad != '' and $this->newdir != '' and $this->olddir != '') {
            $olddir = $this->pfad . '/bilder/' . image_upload::key_url($this->olddir);
            $newdir = $this->pfad . '/bilder/' . image_upload::key_url($this->newdir);
            if (@is_dir($olddir)) {
                return @rename($olddir, $newdir);
            }

        }
        return false;
    }

    /**
     * @return bool Returns true on success or false on failure
     */
    function rmdir_dir(): bool
    {
        if ($this->rmdir_dir) {
            return (@rmdir($this->rmdir_dir));
        }
        return false;
    }

    /**
     * @return string
     */
    function _fileupload(): string
    {
        global $user_id, $dbh;
        if ($_FILES['this_file']['size'] > $this->maxsize) {
            unset($_FILES['this_file']['tmp_name']);
            $this->error(1, $_FILES['this_file']['size'] . " Bytes");
        }
        $i = 0;
        if (is_array($this->alowfile)) {
            foreach ($this->alowfile as $key => $value) {
                if ($_FILES['this_file']['type'] == $value) {
                    $i++;
                }
            }
        }
        if ($i == 0) {
            unset($_FILES['this_file']['tmp_name']);
            $this->error(2, $_FILES['this_file']['type']);
        } else {
            if (!$this->pfad) {
                $this->error(0, "Keine g&uuml;ltige Pfadangabe. ");
            } else {
                /**
                 * Pr&uuml;fen ob Kategorie Eintrag gibt in der Datenbank
                 */
                $sql_select = "SELECT kategorie_dir  FROM `kategorie` WHERE `id`='" . $dbh->escape($this->kategorie_id) . "' LIMIT 1;";
                $sth = $dbh->prepare($sql_select, '-1');
                $sth->execute();
                $daten = $sth->fetchrow_hash();


                if($daten===null && $this->kategorie_id===0){
                    $path_to_kategorie = $this->pfad . '/' ;
                } else {
                    $path_to_kategorie = $this->pfad . '/' . $daten['kategorie_dir'] . '/';
                }
                /**
                 * Pr&uuml;fen ob den Kategorie Ordner auf den Server gibt
                 */
                if (!@is_dir($path_to_kategorie)) {
                    $this->error(5, "Keine g&uuml;ltige Pfadangabe. Bitte pr&uuml;fen Sie ob " . $path_to_kategorie . " ein Verzeichnis ist!");
                } else {
                    $filename = image_upload::key_url($_FILES['this_file']['name']);
                    $image_neu = $path_to_kategorie . $filename;

                    /**
                     * Pr&uuml;fen ob es das Bild schon in der Datenbank gibt
                     */
                    $sql_select = "SELECT count(id) AS `anz` FROM `bilder` WHERE `dateiname` = '" . $dbh->escape($filename) . "' AND kategorie_id='" . $dbh->escape($this->kategorie_id) . "' LIMIT 1;";
                    $sth = $dbh->prepare($sql_select, '-1');
                    $sth->execute();
                    $daten = $sth->fetchrow_hash();
                    if (!isset($daten['anz'])) $anz = 0; else $anz = $daten['anz'];
                    unset($daten);
                    $sth->finish();
                    $t = 0;
                    if ($anz != 0) {
                        if (!@file_exists($image_neu)) {
                            $sql_delete = "DELETE FROM `bilder` WHERE `dateiname` = '" . $dbh->escape($filename) . "' AND kategorie_id='" . $dbh->escape($this->kategorie_id) . "' LIMIT 1;";
                            $sth = $dbh->prepare($sql_delete);
                            $sth->execute();
                        } else {
                            $this->error(4, "");
                            $t = 1;
                        }
                    }
                    if ($t == 0) {
                        if (decoct(fileperms($this->pfad)) == 40755 or decoct(fileperms($this->pfad)) == 40777) {
                            if (@move_uploaded_file($_FILES['this_file']['tmp_name'], $image_neu)) {
                                $sql_insert = "INSERT INTO `bilder` (`dateiname`,`kategorie_id`,`timestamp`,`admin_id`, `beschreibung`) VALUES(
			                    		'" . $dbh->escape($filename) . "', '" . $dbh->escape($this->kategorie_id) . "','" . time() . "','" . $user_id . "','" . $dbh->escape($this->beschreibung) . "');";
                                $sth = $dbh->prepare($sql_insert);
                                $sth->execute();
                                unset($_FILES['this_file']);
                                @chmod($image_neu, 0755);
                                return '1';
                            } else {
                                $this->error(0, "Keine g&uuml;ltige Pfadangabe oder keine Schreibrechte.");
                            }
                        } else {
                            $this->error(3, "Der Upload Ordner hat keine Schreibrechte.");
                        }
                    }
                }
            }
        }
        return '';
    }

}